@@ -32,26 +32,22 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_BT_HAVE_NONE_MESSAGE_H_
-#define _D_BT_HAVE_NONE_MESSAGE_H_
+#ifndef D_BT_HAVE_NONE_MESSAGE_H
+#define D_BT_HAVE_NONE_MESSAGE_H
 
 #include "ZeroBtMessage.h"
 
 namespace aria2 {
 
-class BtHaveNoneMessage;
-
-typedef SharedHandle<BtHaveNoneMessage> BtHaveNoneMessageHandle;
-
 class BtHaveNoneMessage : public ZeroBtMessage {
 public:
   BtHaveNoneMessage();
 
   static const uint8_t ID = 15;
 
-  static const std::string NAME;
+  static const char NAME[];
 
-  static SharedHandle<BtHaveNoneMessage> create
+  static BtHaveNoneMessage* create
   (const unsigned char* data, size_t dataLength);
 
   virtual void doReceivedAction();
@@ -59,4 +55,4 @@ public:
 
 } // namespace aria2
 
-#endif // _D_BT_HAVE_NONE_MESSAGE_H_
+#endif // D_BT_HAVE_NONE_MESSAGE_H
