@@ -54,24 +54,28 @@ private:
   SharedHandle<BtRuntime> btRuntime_;
   SharedHandle<PieceStorage> pieceStorage_;
   size_t maxPeerListSize_;
-  std::deque<SharedHandle<Peer> > peers_;
+
+  // This contains ip address and port pair and is used to ensure that
+  // no duplicate peers are stored.
+  std::set<std::pair<std::string, uint16_t> > uniqPeers_;
+  // Unused (not connected) peers, sorted by last added.
+  std::deque<SharedHandle<Peer> > unusedPeers_;
+  // The set of used peers. Some of them are not connected yet. To
+  // know it is connected or not, call Peer::isActive().
+  PeerSet usedPeers_;
+
   std::deque<SharedHandle<Peer> > droppedPeers_;
-  int64_t removedPeerSessionDownloadLength_;
-  int64_t removedPeerSessionUploadLength_;
 
   BtSeederStateChoke* seederStateChoke_;
   BtLeecherStateChoke* leecherStateChoke_;
 
-  std::map<std::string, TransferStat> peerTransferStatMap_;
-
   Timer lastTransferStatMapUpdated_;
 
-  TransferStat cachedTransferStat_;
-
   std::map<std::string, time_t> badPeers_;
   Timer lastBadPeerCleaned_;
 
   bool isPeerAlreadyAdded(const SharedHandle<Peer>& peer);
+  void addUniqPeer(const SharedHandle<Peer>& peer);
 
   void addDroppedPeer(const SharedHandle<Peer>& peer);
 public:
@@ -79,34 +83,29 @@ public:
 
   virtual ~DefaultPeerStorage();
 
+  // TODO We need addAndCheckoutPeer for incoming peers
   virtual bool addPeer(const SharedHandle<Peer>& peer);
 
-  virtual size_t countPeer() const;
-
-  virtual SharedHandle<Peer> getUnusedPeer();
+  virtual size_t countAllPeer() const;
 
   SharedHandle<Peer> getPeer(const std::string& ipaddr, uint16_t port) const;
 
   virtual void addPeer(const std::vector<SharedHandle<Peer> >& peers);
 
-  virtual const std::deque<SharedHandle<Peer> >& getPeers();
+  const std::deque<SharedHandle<Peer> >& getUnusedPeers();
+
+  virtual const PeerSet& getUsedPeers();
 
   virtual const std::deque<SharedHandle<Peer> >& getDroppedPeers();
 
   virtual bool isPeerAvailable();
 
-  virtual void getActivePeers(std::vector<SharedHandle<Peer> >& peers);
-
-  virtual TransferStat calculateStat();
-
-  virtual void updateTransferStatFor(const SharedHandle<Peer>& peer);
-
-  virtual TransferStat getTransferStatFor(const SharedHandle<Peer>& peer);
-
   virtual bool isBadPeer(const std::string& ipaddr);
 
   virtual void addBadPeer(const std::string& ipaddr);
 
+  virtual SharedHandle<Peer> checkoutPeer(cuid_t cuid);
+
   virtual void returnPeer(const SharedHandle<Peer>& peer);
 
   virtual bool chokeRoundIntervalElapsed();
@@ -114,7 +113,7 @@ public:
   virtual void executeChoke();
 
   void deleteUnusedPeer(size_t delSize);
-  
+
   void onErasingPeer(const SharedHandle<Peer>& peer);
 
   void onReturningPeer(const SharedHandle<Peer>& peer);
