@@ -32,8 +32,8 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_NETRC_AUTH_RESOLVER_H_
-#define _D_NETRC_AUTH_RESOLVER_H_
+#ifndef D_NETRC_AUTH_RESOLVER_H
+#define D_NETRC_AUTH_RESOLVER_H
 
 #include "AbstractAuthResolver.h"
 
@@ -43,34 +43,32 @@ class Netrc;
 
 class NetrcAuthResolver : public AbstractAuthResolver {
 private:
-  SharedHandle<Netrc> _netrc;
+  SharedHandle<Netrc> netrc_;
 
-  bool _ignoreDefault;
+  bool ignoreDefault_;
 
-  SharedHandle<AuthConfig> findNetrcAuthenticator(const std::string& hostname) const;
+  SharedHandle<AuthConfig> findNetrcAuthenticator
+  (const std::string& hostname) const;
 public:
   NetrcAuthResolver();
 
-  virtual ~NetrcAuthResolver() {}
-
-  virtual SharedHandle<AuthConfig> resolveAuthConfig(const std::string& hostname);
+  virtual SharedHandle<AuthConfig> resolveAuthConfig
+  (const std::string& hostname);
 
   void setNetrc(const SharedHandle<Netrc>& netrc);
 
   const SharedHandle<Netrc>& getNetrc() const
   {
-    return _netrc;
+    return netrc_;
   }
 
   // Ignores default token of netrc
   void ignoreDefault();
-  
+
   // Uses default token of netrc
   void useDefault();
 };
 
-typedef SharedHandle<NetrcAuthResolver> NetrcAuthResolverHandle;
-
 } // namespace aria2
 
-#endif // _D_NETRC_AUTH_RESOLVER_H_
+#endif // D_NETRC_AUTH_RESOLVER_H
