@@ -42,25 +42,25 @@
 
 namespace aria2 {
 
-Time::Time():_good(true)
+Time::Time():good_(true)
 {
   reset();
 }
 
 Time::Time(const Time& time)
 {
-  tv = time.tv;
-  _good = time._good;
+  tv_ = time.tv_;
+  good_ = time.good_;
 }
 
-Time::Time(time_t sec):_good(true)
+Time::Time(time_t sec):good_(true)
 {
   setTimeInSec(sec);
 }
 
-Time::Time(const struct timeval& tv):_good(true)
+Time::Time(const struct timeval& tv):good_(true)
 {
-  this->tv = tv;
+  tv_ = tv;
 }
 
 Time::~Time() {}
@@ -68,19 +68,19 @@ Time::~Time() {}
 Time& Time::operator=(const Time& time)
 {
   if(this != &time) {
-    tv = time.tv;
-    _good = time._good;
+    tv_ = time.tv_;
+    good_ = time.good_;
   }
   return *this;
 }
 
 bool Time::operator<(const Time& time) const
 {
-  return util::difftv(time.tv, tv) > 0;
+  return util::difftv(time.tv_, tv_) > 0;
 }
 
 void Time::reset() {
-  gettimeofday(&tv, 0);
+  gettimeofday(&tv_, 0);
 }
 
 struct timeval Time::getCurrentTime() const {
@@ -95,81 +95,90 @@ bool Time::elapsed(time_t sec) const {
   // simple test using time.
   // Then only when the further test is required, call gettimeofday.
   time_t now = time(0);
-  if(tv.tv_sec+sec < now) {
+  if(tv_.tv_sec+sec < now) {
     return true;
-  } else if(tv.tv_sec+sec == now) {
+  } else if(tv_.tv_sec+sec == now) {
     return
-      util::difftv(getCurrentTime(), tv) >= static_cast<int64_t>(sec)*1000000;
+      util::difftv(getCurrentTime(), tv_) >= static_cast<int64_t>(sec)*1000000;
   } else {
     return false;
   }
 }
 
 bool Time::elapsedInMillis(int64_t millis) const {
-  return util::difftv(getCurrentTime(), tv)/1000 >= millis;
+  return util::difftv(getCurrentTime(), tv_)/1000 >= millis;
 }
 
 bool Time::isNewer(const Time& time) const {
-  return util::difftv(this->tv, time.tv) > 0;
+  return util::difftv(tv_, time.tv_) > 0;
 }
 
 time_t Time::difference() const
 {
-  return util::difftv(getCurrentTime(), tv)/1000000;
+  return util::difftv(getCurrentTime(), tv_)/1000000;
 }
 
 time_t Time::difference(const struct timeval& now) const
 {
-  return util::difftv(now, tv)/1000000;
+  return util::difftv(now, tv_)/1000000;
 }
 
 int64_t Time::differenceInMillis() const {
-  return util::difftv(getCurrentTime(), tv)/1000;
+  return util::difftv(getCurrentTime(), tv_)/1000;
 }
 
 int64_t Time::differenceInMillis(const struct timeval& now) const
 {
-  return util::difftv(now, tv)/1000;
+  return util::difftv(now, tv_)/1000;
 }
 
 bool Time::isZero() const
 {
-  return tv.tv_sec == 0 && tv.tv_usec == 0;
+  return tv_.tv_sec == 0 && tv_.tv_usec == 0;
 }
 
 int64_t Time::getTimeInMicros() const
 {
-  return (int64_t)tv.tv_sec*1000*1000+tv.tv_usec;
+  return (int64_t)tv_.tv_sec*1000*1000+tv_.tv_usec;
 }
 
 int64_t Time::getTimeInMillis() const
 {
-  return (int64_t)tv.tv_sec*1000+tv.tv_usec/1000;
+  return (int64_t)tv_.tv_sec*1000+tv_.tv_usec/1000;
 }
 
 time_t Time::getTime() const
 {
-  return tv.tv_sec;
+  return tv_.tv_sec;
 }
 
 void Time::setTimeInSec(time_t sec) {
-  tv.tv_sec = sec;
-  tv.tv_usec = 0;
+  tv_.tv_sec = sec;
+  tv_.tv_usec = 0;
 }
 
 void Time::advance(time_t sec)
 {
-  tv.tv_sec += sec;
+  tv_.tv_sec += sec;
 }
 
 bool Time::good() const
 {
-  return _good;
+  return good_;
 }
 
 bool Time::bad() const
 {
-  return !_good;
+  return !good_;
+}
+
+std::string Time::toHTTPDate() const
+{
+  char buf[32];
+  time_t t = getTime();
+  struct tm* tms = gmtime(&t); // returned struct is statically allocated.
+  size_t r = strftime(buf, sizeof(buf), "%a, %d %b %Y %H:%M:%S GMT", tms);
+  return std::string(&buf[0], &buf[r]);
 }
 
 Time Time::parse(const std::string& datetime, const std::string& format)
@@ -186,7 +195,7 @@ Time Time::parse(const std::string& datetime, const std::string& format)
       thetime = INT32_MAX;
     }
   }
-  return Time(thetime);  
+  return Time(thetime);
 }
 
 Time Time::parseRFC1123(const std::string& datetime)
@@ -204,11 +213,17 @@ Time Time::parseRFC850Ext(const std::string& datetime)
   return parse(datetime, "%a, %d-%b-%Y %H:%M:%S GMT");
 }
 
+Time Time::parseAsctime(const std::string& datetime)
+{
+  return parse(datetime, "%a %b %d %H:%M:%S %Y");
+}
+
 Time Time::parseHTTPDate(const std::string& datetime)
 {
   Time (*funcs[])(const std::string&) = {
-    &parseRFC850,
     &parseRFC1123,
+    &parseRFC850,
+    &parseAsctime,
     &parseRFC850Ext,
   };
   for(Time (**funcsp)(const std::string&) = &funcs[0];
@@ -224,7 +239,7 @@ Time Time::parseHTTPDate(const std::string& datetime)
 Time Time::null()
 {
   Time t(0);
-  t._good = false;
+  t.good_ = false;
   return t;
 }
 
